# If any of the following macros should be set otherwise,
# you can wrap any of them with the following conditions:
# - %%if 0%%{centos} == 7
# - %%if 0%%{?rhel} == 7
# - %%if 0%%{?fedora} == 23
# Or just test for particular distribution:
# - %%if 0%%{centos}
# - %%if 0%%{?rhel}
# - %%if 0%%{?fedora}
#
# Be aware, on centos, both %%rhel and %%centos are set. If you want to test
# rhel specific macros, you can use %%if 0%%{?rhel} && 0%%{?centos} == 0 condition.
# (Don't forget to replace double percentage symbol with single one in order to apply a condition)

# Generate devel rpm
%global with_devel 1
# Build project from bundled dependencies
%global with_bundled 1
# Build with debug info rpm
%global with_debug 0
# Run tests in check section
%global with_check 1
# Generate unit-test rpm
%global with_unit_test 0

%if 0%{?with_debug}
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif

%if ! 0%{?gobuild:1}
%define gobuild(o:) go build -ldflags "${LDFLAGS:-} -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n')" -a -v -x %{?**};
%endif

%global provider        github
%global provider_tld    com
%global project         prometheus
%global repo            alertmanager
# https://github.com/prometheus/alertmanager
%global provider_prefix %{provider}.%{provider_tld}/%{project}/%{repo}
%global import_path     %{provider_prefix}
%global commit          133c888ef3644b47a52acbaeffb09f4cc637df1b
%global shortcommit     %(c=%{commit}; echo ${c:0:7})

Name:           golang-%{provider}-%{project}-%{repo}
Version:        0.10.0
Release:        1.git%{shortcommit}%{?dist}
Summary:        The Prometheus Alertmanager handles alerts sent by client applications.
License:        ASL 2.0
URL:            https://%{provider_prefix}
Source0:        https://%{provider_prefix}/archive/%{commit}/%{repo}-%{shortcommit}.tar.gz

# e.g. el6 has ppc64 arch without gcc-go, so EA tag is required
ExclusiveArch:  %{?go_arches:%{go_arches}}%{!?go_arches:%{ix86} x86_64 aarch64 %{arm}}
# If go_compiler is not set to 1, there is no virtual provide. Use golang instead.
BuildRequires:  %{?go_compiler:compiler(go-compiler)}%{!?go_compiler:golang}

%if ! 0%{?with_bundled}
# cmd/alertmanager/main.go
BuildRequires: golang(github.com/go-kit/kit/log)
BuildRequires: golang(github.com/go-kit/kit/log/level)
BuildRequires: golang(github.com/prometheus/client_golang/prometheus)
BuildRequires: golang(github.com/prometheus/common/promlog)
BuildRequires: golang(github.com/prometheus/common/route)
BuildRequires: golang(github.com/prometheus/common/version)
BuildRequires: golang(github.com/prometheus/prometheus/pkg/labels)
BuildRequires: golang(github.com/weaveworks/mesh)

# artifacts/generate_amtool_artifacts.go
BuildRequires: golang(github.com/spf13/cobra/doc)

# Remaining dependencies not included in main packages
BuildRequires: golang(github.com/spf13/cobra)
BuildRequires: golang(golang.org/x/net/context/ctxhttp)
BuildRequires: golang(github.com/gogo/protobuf/proto)
BuildRequires: golang(gopkg.in/yaml.v2)
BuildRequires: golang(github.com/prometheus/client_golang/api/alertmanager)
BuildRequires: golang(github.com/oklog/oklog/pkg/group)
BuildRequires: golang(github.com/satori/go.uuid)
BuildRequires: golang(golang.org/x/net/context)
BuildRequires: golang(github.com/spf13/pflag)
BuildRequires: golang(github.com/pkg/errors)
BuildRequires: golang(github.com/gogo/protobuf/types)
BuildRequires: golang(github.com/matttproud/golang_protobuf_extensions/pbutil)
BuildRequires: golang(github.com/prometheus/common/model)
BuildRequires: golang(github.com/cespare/xxhash)
BuildRequires: golang(github.com/spf13/viper)
BuildRequires: golang(github.com/cenkalti/backoff)
%endif

%description
%{summary}

# binary package
%package -n %{project}-%{repo}
Summary:       %{summary}
Provides:      %{repo} = %{version}-%{release}

%description -n %{project}-%{repo}
%{summary}

%if 0%{?with_devel}
%package devel
Summary:       %{summary}
BuildArch:     noarch

%if 0%{?with_check} && ! 0%{?with_bundled}
BuildRequires: golang(github.com/cenkalti/backoff)
BuildRequires: golang(github.com/cespare/xxhash)
BuildRequires: golang(github.com/go-kit/kit/log)
BuildRequires: golang(github.com/go-kit/kit/log/level)
BuildRequires: golang(github.com/gogo/protobuf/proto)
BuildRequires: golang(github.com/gogo/protobuf/types)
BuildRequires: golang(github.com/matttproud/golang_protobuf_extensions/pbutil)
BuildRequires: golang(github.com/oklog/oklog/pkg/group)
BuildRequires: golang(github.com/pkg/errors)
BuildRequires: golang(github.com/prometheus/client_golang/api/alertmanager)
BuildRequires: golang(github.com/prometheus/client_golang/prometheus)
BuildRequires: golang(github.com/prometheus/common/model)
BuildRequires: golang(github.com/prometheus/common/route)
BuildRequires: golang(github.com/prometheus/common/version)
BuildRequires: golang(github.com/prometheus/prometheus/pkg/labels)
BuildRequires: golang(github.com/satori/go.uuid)
BuildRequires: golang(github.com/spf13/cobra)
BuildRequires: golang(github.com/spf13/pflag)
BuildRequires: golang(github.com/spf13/viper)
BuildRequires: golang(github.com/weaveworks/mesh)
BuildRequires: golang(golang.org/x/net/context)
BuildRequires: golang(golang.org/x/net/context/ctxhttp)
BuildRequires: golang(gopkg.in/yaml.v2)
%endif

Requires:      golang(github.com/cenkalti/backoff)
Requires:      golang(github.com/cespare/xxhash)
Requires:      golang(github.com/go-kit/kit/log)
Requires:      golang(github.com/go-kit/kit/log/level)
Requires:      golang(github.com/gogo/protobuf/proto)
Requires:      golang(github.com/gogo/protobuf/types)
Requires:      golang(github.com/matttproud/golang_protobuf_extensions/pbutil)
Requires:      golang(github.com/oklog/oklog/pkg/group)
Requires:      golang(github.com/pkg/errors)
Requires:      golang(github.com/prometheus/client_golang/api/alertmanager)
Requires:      golang(github.com/prometheus/client_golang/prometheus)
Requires:      golang(github.com/prometheus/common/model)
Requires:      golang(github.com/prometheus/common/route)
Requires:      golang(github.com/prometheus/common/version)
Requires:      golang(github.com/prometheus/prometheus/pkg/labels)
Requires:      golang(github.com/satori/go.uuid)
Requires:      golang(github.com/spf13/cobra)
Requires:      golang(github.com/spf13/pflag)
Requires:      golang(github.com/spf13/viper)
Requires:      golang(github.com/weaveworks/mesh)
Requires:      golang(golang.org/x/net/context)
Requires:      golang(golang.org/x/net/context/ctxhttp)
Requires:      golang(gopkg.in/yaml.v2)

Provides:      golang(%{import_path}/api) = %{version}-%{release}
Provides:      golang(%{import_path}/cli) = %{version}-%{release}
Provides:      golang(%{import_path}/cli/format) = %{version}-%{release}
Provides:      golang(%{import_path}/config) = %{version}-%{release}
Provides:      golang(%{import_path}/dispatch) = %{version}-%{release}
Provides:      golang(%{import_path}/inhibit) = %{version}-%{release}
Provides:      golang(%{import_path}/nflog) = %{version}-%{release}
Provides:      golang(%{import_path}/nflog/nflogpb) = %{version}-%{release}
Provides:      golang(%{import_path}/notify) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/parse) = %{version}-%{release}
Provides:      golang(%{import_path}/provider) = %{version}-%{release}
Provides:      golang(%{import_path}/provider/mem) = %{version}-%{release}
Provides:      golang(%{import_path}/silence) = %{version}-%{release}
Provides:      golang(%{import_path}/silence/silencepb) = %{version}-%{release}
Provides:      golang(%{import_path}/template) = %{version}-%{release}
Provides:      golang(%{import_path}/test) = %{version}-%{release}
Provides:      golang(%{import_path}/types) = %{version}-%{release}
Provides:      golang(%{import_path}/ui) = %{version}-%{release}

%description devel
%{summary}

This package contains library source intended for
building other packages which use import path with
%{import_path} prefix.
%endif

%if 0%{?with_unit_test} && 0%{?with_devel}
%package unit-test-devel
Summary:         Unit tests for %{name} package
%if 0%{?with_check}
#Here comes all BuildRequires: PACKAGE the unit tests
#in %%check section need for running
%endif

# test subpackage tests code from devel subpackage
Requires:        %{name}-devel = %{version}-%{release}

%if 0%{?with_check} && ! 0%{?with_bundled}
BuildRequires: golang(github.com/golang/protobuf/ptypes)
BuildRequires: golang(github.com/golang/protobuf/ptypes/timestamp)
BuildRequires: golang(github.com/kylelemons/godebug/pretty)
BuildRequires: golang(github.com/stretchr/testify/require)
%endif

Requires:      golang(github.com/golang/protobuf/ptypes)
Requires:      golang(github.com/golang/protobuf/ptypes/timestamp)
Requires:      golang(github.com/kylelemons/godebug/pretty)
Requires:      golang(github.com/stretchr/testify/require)

%description unit-test-devel
%{summary}

This package contains unit tests for project
providing packages with %{import_path} prefix.
%endif

%prep
%setup -q -n %{repo}-%{commit}

%build
mkdir -p src/%{provider}.%{provider_tld}/%{project}
ln -s ../../../ src/%{import_path}

%if ! 0%{?with_bundled}
export GOPATH=$(pwd):%{gopath}
%else
# No dependency directories so far
export GOPATH=$(pwd):%{gopath}
%endif

%gobuild -o bin/artifacts %{import_path}/artifacts
%gobuild -o bin/cmd/alertmanager %{import_path}/cmd/alertmanager
%gobuild -o bin/cmd/amtool %{import_path}/cmd/amtool
%gobuild -o bin/examples/webhook %{import_path}/examples/webhook

%install
install -d -p %{buildroot}%{_bindir}
install -p -m 0755 bin/cmd/alertmanager %{buildroot}%{_bindir}/prometheus-alertmanager
install -p -m 0755 bin/cmd/amtool %{buildroot}%{_bindir}/prometheus-amtool

# source codes for building projects
%if 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
echo "%%dir %%{gopath}/src/%%{import_path}/." >> devel.file-list
# find all *.go but no *_test.go files and generate devel.file-list
for file in $(find . \( -iname "*.go" -or -iname "*.s" \) \! -iname "*_test.go" | grep -v "vendor") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

# testing files for this project
%if 0%{?with_unit_test} && 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
# find all *_test.go files and generate unit-test-devel.file-list
for file in $(find . -iname "*_test.go" | grep -v "vendor") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> unit-test-devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

%if 0%{?with_devel}
sort -u -o devel.file-list devel.file-list
%endif

%check
%if 0%{?with_check} && 0%{?with_unit_test} && 0%{?with_devel}
%if ! 0%{?with_bundled}
export GOPATH=%{buildroot}/%{gopath}:%{gopath}
%else
# Since we aren't packaging up the vendor directory we need to link
# back to it somehow. Hack it up so that we can add the vendor
# directory from BUILD dir as a gopath to be searched when executing
# tests from the BUILDROOT dir.
ln -s ./ ./vendor/src # ./vendor/src -> ./vendor

export GOPATH=%{buildroot}/%{gopath}:$(pwd)/vendor:%{gopath}
%endif

%if ! 0%{?gotest:1}
%global gotest go test
%endif

%gotest %{import_path}/api
%gotest %{import_path}/cli
%gotest %{import_path}/config
%gotest %{import_path}/dispatch
%gotest %{import_path}/inhibit
%gotest %{import_path}/nflog
%gotest %{import_path}/nflog/nflogpb
%gotest %{import_path}/notify
%gotest %{import_path}/pkg/parse
%gotest %{import_path}/provider/mem
%gotest %{import_path}/silence
%gotest %{import_path}/test/acceptance
%gotest %{import_path}/types
%endif

#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files -n %{project}-%{repo}
%license LICENSE NOTICE
%doc CHANGELOG.md MAINTAINERS.md CONTRIBUTING.md README.md
%{_bindir}/prometheus-alertmanager
%{_bindir}/prometheus-amtool

%if 0%{?with_devel}
%files devel -f devel.file-list
%license LICENSE
%doc CHANGELOG.md MAINTAINERS.md CONTRIBUTING.md README.md
%dir %{gopath}/src/%{provider}.%{provider_tld}/%{project}
%endif

%if 0%{?with_unit_test} && 0%{?with_devel}
%files unit-test-devel -f unit-test-devel.file-list
%license LICENSE
%doc CHANGELOG.md MAINTAINERS.md CONTRIBUTING.md README.md
%endif

%changelog
* Wed Nov 15 2017 Paul Gier <pgier@redhat.com> - 0.10.0-1.git133c888
- First package for Fedora

